@@ -46,14 +46,14 @@
 #include "HttpRequest.h"
 #include "forward.h"
 #include "clientStream.h"
-#include "AuthUserRequest.h"
+#include "auth/UserRequest.h"
 #if USE_SQUID_ESI
-#include "ESI.h"
+#include "esi/Esi.h"
 #endif
 #include "MemObject.h"
 #include "fde.h"
-#include "ACLChecklist.h"
-#include "ACL.h"
+#include "acl/FilledChecklist.h"
+#include "acl/Gadgets.h"
 #if DELAY_POOLS
 #include "DelayPools.h"
 #endif
@@ -86,8 +86,9 @@ clientReplyContext::~clientReplyContext()
 clientReplyContext::clientReplyContext(ClientHttpRequest *clientContext) : http (cbdataReference(clientContext)), old_entry (NULL), old_sc(NULL), deleting(false)
 {}
 
-/* create an error in the store awaiting the client side to read it. */
-/* This may be better placed in the clientStream logic, but it has not been
+/** Create an error in the store awaiting the client side to read it.
+ *
+ * This may be better placed in the clientStream logic, but it has not been
  * relocated there yet
  */
 void
@@ -649,10 +650,9 @@ clientReplyContext::processMiss()
         return;
     }
 
-    /**
-     * Deny loops when running in accelerator/transproxy mode.
-     */
-    if (http->flags.accel && r->flags.loopdetect) {
+    /// Deny loops for accelerator and interceptor. TODO: deny in all modes?
+    if (r->flags.loopdetect &&
+        (http->flags.accel || http->flags.intercepted)) {
         http->al.http.code = HTTP_FORBIDDEN;
         err = clientBuildError(ERR_ACCESS_DENIED, HTTP_FORBIDDEN, NULL, http->getConn()->peer, http->request);
         createStoreEntry(r->method, request_flags());
@@ -1199,10 +1199,12 @@ clientReplyContext::alwaysAllowResponse(http_status sline) const
     return result;
 }
 
-/*
- * filters out unwanted entries from original reply header
- * adds extra entries if we have more info than origin server
- * adds Squid specific entries
+/**
+ * Generate the reply headers sent to client.
+ *
+ * Filters out unwanted entries and hop-by-hop from original reply header
+ * then adds extra entries if we have more info than origin server
+ * then adds Squid specific entries
  */
 void
 clientReplyContext::buildReplyHeader()
@@ -1228,6 +1230,7 @@ clientReplyContext::buildReplyHeader()
 
     //    if (request->range)
     //      clientBuildRangeHeader(http, reply);
+
     /*
      * Add a estimated Age header on cache hits.
      */
@@ -1249,11 +1252,14 @@ clientReplyContext::buildReplyHeader()
          * the objects age, so a Age: 0 header does not add any useful
          * information to the reply in any case.
          */
-
+#if DEAD_CODE
+        // XXX: realy useless? or is there a bug now that this is detatched from the below if-sequence ?
+        // looks like this pre-if was supposed to be the browser workaround...
         if (NULL == http->storeEntry())
             (void) 0;
         else if (http->storeEntry()->timestamp < 0)
             (void) 0;
+#endif
 
         if (EBIT_TEST(http->storeEntry()->flags, ENTRY_SPECIAL)) {
             hdr->delById(HDR_DATE);
@@ -1278,7 +1284,24 @@ clientReplyContext::buildReplyHeader()
                 hdr->putStr(HDR_WARNING, tempbuf);
             }
         }
+    }
 
+    /* RFC 2616: Section 14.18
+     *
+     * Add a Date: header if missing.
+     * We have access to a clock therefore are required to amend any shortcoming in servers.
+     *
+     * NP: done after Age: to prevent ENTRY_SPECIAL double-handling this header.
+     */
+    if ( !hdr->has(HDR_DATE) ) {
+        if (!http->storeEntry())
+            hdr->insertTime(HDR_DATE, squid_curtime);
+        else if (http->storeEntry()->timestamp > 0)
+            hdr->insertTime(HDR_DATE, http->storeEntry()->timestamp);
+        else {
+            debugs(88,1,"WARNING: An error inside Squid has caused an HTTP reply without Date:. Please report this");
+            /* TODO: dump something useful about the problem */
+        }
     }
 
     /* Filter unproxyable authentication types */
@@ -1357,24 +1380,19 @@ clientReplyContext::buildReplyHeader()
     if (!Config.onoff.error_pconns && reply->sline.status >= 400 && !request->flags.must_keepalive) {
         debugs(33, 3, "clientBuildReplyHeader: Error, don't keep-alive");
         request->flags.proxy_keepalive = 0;
-    }
-    else if (!Config.onoff.client_pconns && !request->flags.must_keepalive) {
+    } else if (!Config.onoff.client_pconns && !request->flags.must_keepalive) {
         debugs(33, 2, "clientBuildReplyHeader: Connection Keep-Alive not requested by admin or client");
         request->flags.proxy_keepalive = 0;
-    }
-    else if (request->flags.proxy_keepalive && shutting_down) {
+    } else if (request->flags.proxy_keepalive && shutting_down) {
         debugs(88, 3, "clientBuildReplyHeader: Shutting down, don't keep-alive.");
         request->flags.proxy_keepalive = 0;
-    }
-    else if (request->flags.connection_auth && !reply->keep_alive) {
+    } else if (request->flags.connection_auth && !reply->keep_alive) {
         debugs(33, 2, "clientBuildReplyHeader: Connection oriented auth but server side non-persistent");
         request->flags.proxy_keepalive = 0;
-    }
-    else if (reply->bodySize(request->method) < 0) {
+    } else if (reply->bodySize(request->method) < 0) {
         debugs(88, 3, "clientBuildReplyHeader: can't keep-alive, unknown body size" );
         request->flags.proxy_keepalive = 0;
-    }
-    else if (fdUsageHigh()&& !request->flags.must_keepalive) {
+    } else if (fdUsageHigh()&& !request->flags.must_keepalive) {
         debugs(88, 3, "clientBuildReplyHeader: Not many unused FDs, can't keep-alive");
         request->flags.proxy_keepalive = 0;
     }
@@ -1385,7 +1403,7 @@ clientReplyContext::buildReplyHeader()
         LOCAL_ARRAY(char, bbuf, MAX_URL + 32);
         String strVia;
         hdr->getList(HDR_VIA, &strVia);
-        snprintf(bbuf, sizeof(bbuf), "%d.%d %s",
+        snprintf(bbuf, MAX_URL + 32, "%d.%d %s",
                  reply->sline.version.major,
                  reply->sline.version.minor,
                  ThisCache);
@@ -1452,7 +1470,7 @@ clientReplyContext::identifyFoundObject(StoreEntry *newEntry)
     StoreEntry *e = newEntry;
     HttpRequest *r = http->request;
 
-    /** \item  If the entry received isNull() then we ignore it. */
+    /** \li If the entry received isNull() then we ignore it. */
     if (e->isNull()) {
         http->storeEntry(NULL);
     } else {
@@ -1462,7 +1480,7 @@ clientReplyContext::identifyFoundObject(StoreEntry *newEntry)
     e = http->storeEntry();
 
     /* Release IP-cache entries on reload */
-    /** \item If the request has no-cache flag set or some no_cache HACK in operation we
+    /** \li If the request has no-cache flag set or some no_cache HACK in operation we
       * 'invalidate' the cached IP entries for this request ???
       */
     if (r->flags.nocache) {
@@ -1503,7 +1521,7 @@ clientReplyContext::identifyFoundObject(StoreEntry *newEntry)
 #endif
 
     if (NULL == http->storeEntry()) {
-        /** \item If no StoreEntry object is current assume this object isn't in the cache set MISS*/
+        /** \li If no StoreEntry object is current assume this object isn't in the cache set MISS*/
         debugs(85, 3, "clientProcessRequest2: StoreEntry is NULL -  MISS");
         http->logType = LOG_TCP_MISS;
         doGetMoreData();
@@ -1511,7 +1529,7 @@ clientReplyContext::identifyFoundObject(StoreEntry *newEntry)
     }
 
     if (Config.onoff.offline) {
-        /** \item If we are running in offline mode set to HIT */
+        /** \li If we are running in offline mode set to HIT */
         debugs(85, 3, "clientProcessRequest2: offline HIT");
         http->logType = LOG_TCP_HIT;
         doGetMoreData();
@@ -1519,7 +1537,7 @@ clientReplyContext::identifyFoundObject(StoreEntry *newEntry)
     }
 
     if (http->redirect.status) {
-        /** \item If redirection status is True force this to be a MISS */
+        /** \li If redirection status is True force this to be a MISS */
         debugs(85, 3, "clientProcessRequest2: redirectStatus forced StoreEntry to NULL -  MISS");
         http->storeEntry(NULL);
         http->logType = LOG_TCP_MISS;
@@ -1536,7 +1554,7 @@ clientReplyContext::identifyFoundObject(StoreEntry *newEntry)
     }
 
     if (EBIT_TEST(e->flags, ENTRY_SPECIAL)) {
-        /* Special entries are always hits, no matter what the client says */
+        /* \li Special entries are always hits, no matter what the client says */
         debugs(85, 3, "clientProcessRequest2: ENTRY_SPECIAL HIT");
         http->logType = LOG_TCP_HIT;
         doGetMoreData();
@@ -1556,13 +1574,14 @@ clientReplyContext::identifyFoundObject(StoreEntry *newEntry)
     doGetMoreData();
 }
 
-/* Request more data from the store for the client Stream
+/**
+ * Request more data from the store for the client Stream
  * This is *the* entry point to this module.
  *
  * Preconditions:
- * This is the head of the list.
- * There is at least one more node.
- * data context is not null
+ *  - This is the head of the list.
+ *  - There is at least one more node.
+ *  - Data context is not null
  */
 void
 clientGetMoreData(clientStreamNode * aNode, ClientHttpRequest * http)
@@ -1599,6 +1618,8 @@ clientGetMoreData(clientStreamNode * aNode, ClientHttpRequest * http)
         return;
     }
 
+    /* TODO: handle OPTIONS request on max_forwards == 0 as well */
+
     if (context->http->request->method == METHOD_TRACE) {
         if (context->http->request->max_forwards == 0) {
             context->traceReply(aNode);
@@ -1662,16 +1683,16 @@ clientReplyContext::doGetMoreData()
     }
 }
 
-/* the next node has removed itself from the stream. */
+/** The next node has removed itself from the stream. */
 void
 clientReplyDetach(clientStreamNode * node, ClientHttpRequest * http)
 {
-    /* detach from the stream */
+    /** detach from the stream */
     clientStreamDetach(node, http);
 }
 
-/*
- * accepts chunk of a http message in buf, parses prefix, filters headers and
+/**
+ * Accepts chunk of a http message in buf, parses prefix, filters headers and
  * such, writes processed message to the message recipient
  */
 void
@@ -1701,8 +1722,9 @@ clientReplyContext::errorInStream(StoreIOBuffer const &result, size_t const &siz
 void
 clientReplyContext::sendStreamError(StoreIOBuffer const &result)
 {
-    /* call clientWriteComplete so the client socket gets closed */
-    /* We call into the stream, because we don't know that there is a
+    /** call clientWriteComplete so the client socket gets closed
+     *
+     * We call into the stream, because we don't know that there is a
      * client socket!
      */
     debugs(88, 5, "clientReplyContext::sendStreamError: A stream error has occured, marking as complete and sending no data.");
@@ -1746,6 +1768,7 @@ clientReplyContext::sendBodyTooLargeError()
 {
     IpAddress tmp_noaddr;
     tmp_noaddr.SetNoAddr(); // TODO: make a global const
+    http->logType = LOG_TCP_DENIED_REPLY;
     ErrorState *err = clientBuildError(ERR_TOO_BIG, HTTP_FORBIDDEN, NULL,
                                        http->getConn() != NULL ? http->getConn()->peer : tmp_noaddr,
                                        http->request);
@@ -1785,8 +1808,8 @@ clientReplyContext::processReplyAccess ()
     }
 
     /** Process http_reply_access lists */
-    ACLChecklist *replyChecklist;
-    replyChecklist = clientAclChecklistCreate(Config.accessList.reply, http);
+    ACLFilledChecklist *replyChecklist =
+        clientAclChecklistCreate(Config.accessList.reply, http);
     replyChecklist->reply = HTTPMSGLOCK(reply);
     replyChecklist->nonBlockingCheck(ProcessReplyAccessResult, this);
 }
