@@ -1,6 +1,5 @@
-
 /*
- * $Id: squidclient.cc,v 1.11 2008/02/26 21:49:46 amosjeffries Exp $
+ * $Id$
  *
  * DEBUG: section 0     WWW Client
  * AUTHOR: Harvest Derived
@@ -86,7 +85,7 @@ using namespace Squid;
 #endif
 
 #include "util.h"
-#include "IPAddress.h"
+#include "ip/IpAddress.h"
 
 #ifndef BUFSIZ
 #define BUFSIZ		8192
@@ -101,9 +100,9 @@ using namespace Squid;
 typedef void SIGHDLR(int sig);
 
 /* Local functions */
-static int client_comm_bind(int, const IPAddress &);
+static int client_comm_bind(int, const IpAddress &);
 
-static int client_comm_connect(int, const IPAddress &, struct timeval *);
+static int client_comm_connect(int, const IpAddress &, struct timeval *);
 static void usage(const char *progname);
 
 static int Now(struct timeval *);
@@ -133,27 +132,32 @@ usage(const char *progname)
 {
     fprintf(stderr,
             "Version: %s\n"
-            "Usage: %s [-arsv] [-i IMS] [-h remote host] [-l local host] [-p port] [-m method] [-t count] [-I ping-interval] [-H 'strings'] [-T timeout] url\n"
+            "Usage: %s [-arsv] [-g count] [-h remote host] [-H 'string'] [-i IMS] [-I ping-interval] [-j 'Host-header']"
+            "[-l local-host] [-m method] [-p port] [-P file] [-t count] [-T timeout] [-u proxy-user] [-U www-user] "
+            "[-V version] [-w proxy-password] [-W www-password] url\n"
+            "\n"
             "Options:\n"
-            "    -P file      PUT request.\n"
             "    -a           Do NOT include Accept: header.\n"
+            "    -g count     Ping mode, perform \"count\" iterations (0 to loop until interrupted).\n"
+            "    -h host      Retrieve URL from cache on hostname.  Default is localhost.\n"
+            "    -H 'string'  Extra headers to send. Use '\\n' for new lines.\n"
+            "    -i IMS       If-Modified-Since time (in Epoch seconds).\n"
+            "    -I interval  Ping interval in seconds (default 1 second).\n"
+            "    -j hosthdr   Host header content\n"
+            "    -l host      Specify a local IP address to bind to.  Default is none.\n"
+            "    -m method    Request method, default is GET.\n"
+            "    -p port      Port number of cache.  Default is %d.\n"
+            "    -P file      PUT request. Using the named file\n"
             "    -r           Force cache to reload URL.\n"
             "    -s           Silent.  Do not print data to stdout.\n"
-            "    -v           Verbose. Print outgoing message to stderr.\n"
-            "    -i IMS       If-Modified-Since time (in Epoch seconds).\n"
-            "    -h host      Retrieve URL from cache on hostname.  Default is localhost.\n"
-            "    -l host      Specify a local IP address to bind to.  Default is none.\n"
-            "    -p port      Port number of cache.  Default is %d.\n"
-            "    -m method    Request method, default is GET.\n"
             "    -t count     Trace count cache-hops\n"
-            "    -g count     Ping mode, \"count\" iterations (0 to loop until interrupted).\n"
-            "    -I interval  Ping interval in seconds (default 1 second).\n"
-            "    -H 'string'  Extra headers to send. Use '\\n' for new lines.\n"
             "    -T timeout   Timeout value (seconds) for read/write operations.\n"
             "    -u user      Proxy authentication username\n"
-            "    -w password  Proxy authentication password\n"
             "    -U user      WWW authentication username\n"
-            "    -W password  WWW authentication password\n",
+            "    -v           Verbose. Print outgoing message to stderr.\n"
+            "    -V version   HTTP Version. Use '-' for HTTP/0.9 omitted case\n",
+            "    -w password  Proxy authentication password\n"
+            "    -W password  WWW authentication password\n"
             VERSION, progname, CACHE_HTTP_PORT);
     exit(1);
 }
@@ -169,7 +173,7 @@ main(int argc, char *argv[])
     int opt_noaccept = 0;
     int opt_verbose = 0;
     const char *hostname, *localhost;
-    IPAddress iaddr;
+    IpAddress iaddr;
     char url[BUFSIZ], msg[MESSAGELEN], buf[BUFSIZ];
     char extra_hdrs[HEADERLEN];
     const char *method = "GET";
@@ -181,10 +185,12 @@ main(int argc, char *argv[])
     int i = 0, loops;
     long ping_int;
     long ping_min = 0, ping_max = 0, ping_sum = 0, ping_mean = 0;
-    char *proxy_user = NULL;
-    char *proxy_password = NULL;
-    char *www_user = NULL;
-    char *www_password = NULL;
+    const char *proxy_user = NULL;
+    const char *proxy_password = NULL;
+    const char *www_user = NULL;
+    const char *www_password = NULL;
+    const char *host = NULL;
+    const char *version = "1.0";
 
     /* set the defaults */
     hostname = "localhost";
@@ -206,7 +212,7 @@ main(int argc, char *argv[])
         if (url[0] == '-')
             usage(argv[0]);
 
-        while ((c = getopt(argc, argv, "ah:l:P:i:km:p:rsvt:g:p:I:H:T:u:U:w:W:?")) != -1)
+        while ((c = getopt(argc, argv, "ah:j:V:l:P:i:km:p:rsvt:g:p:I:H:T:u:U:w:W:?")) != -1)
             switch (c) {
 
             case 'a':
@@ -214,83 +220,74 @@ main(int argc, char *argv[])
                 break;
 
             case 'h':		/* remote host */
-
                 if (optarg != NULL)
                     hostname = optarg;
+                break;
 
+            case 'j':
+                host = optarg;
+                break;
+
+            case 'V':
+                if (optarg != NULL)
+                    version = optarg;
                 break;
 
             case 'l':		/* local host */
                 if (optarg != NULL)
                     localhost = optarg;
-
                 break;
 
             case 's':		/* silent */
                 to_stdout = 0;
-
                 break;
 
             case 'k':		/* backward compat */
                 keep_alive = 1;
-
                 break;
 
             case 'r':		/* reload */
                 reload = 1;
-
                 break;
 
             case 'p':		/* port number */
                 sscanf(optarg, "%d", &port);
-
                 if (port < 1)
                     port = CACHE_HTTP_PORT;	/* default */
-
                 break;
 
             case 'P':
                 put_file = xstrdup(optarg);
-
                 break;
 
             case 'i':		/* IMS */
                 ims = (time_t) atoi(optarg);
-
                 break;
 
             case 'm':
                 method = xstrdup(optarg);
-
                 break;
 
             case 't':
                 method = xstrdup("TRACE");
-
                 max_forwards = atoi(optarg);
-
                 break;
 
             case 'g':
                 ping = 1;
-
                 pcount = atoi(optarg);
-
                 to_stdout = 0;
-
                 break;
 
             case 'I':
                 if ((ping_int = atoi(optarg) * 1000) <= 0)
                     usage(argv[0]);
-
                 break;
 
             case 'H':
                 if (strlen(optarg)) {
                     char *t;
                     strncpy(extra_hdrs, optarg, sizeof(extra_hdrs));
-
                     while ((t = strstr(extra_hdrs, "\\n")))
                         *t = '\r', *(t + 1) = '\n';
                 }
@@ -357,78 +354,105 @@ main(int argc, char *argv[])
 
         fstat(put_fd, &sb);
     }
-    snprintf(msg, BUFSIZ, "%s %s HTTP/1.0\r\n", method, url);
 
-    if (reload) {
-        snprintf(buf, BUFSIZ, "Pragma: no-cache\r\n");
-        strcat(msg, buf);
-    }
-    if (put_fd > 0) {
-        snprintf(buf, BUFSIZ, "Content-length: %d\r\n", (int) sb.st_size);
-        strcat(msg, buf);
-    }
-    if (opt_noaccept == 0) {
-        snprintf(buf, BUFSIZ, "Accept: */*\r\n");
-        strcat(msg, buf);
-    }
-    if (ims) {
-        snprintf(buf, BUFSIZ, "If-Modified-Since: %s\r\n", mkrfc1123(ims));
-        strcat(msg, buf);
-    }
-    if (max_forwards > -1) {
-        snprintf(buf, BUFSIZ, "Max-Forwards: %d\r\n", max_forwards);
-        strcat(msg, buf);
-    }
-    if (proxy_user) {
-        char *user = proxy_user;
-        char *password = proxy_password;
-#if HAVE_GETPASS
-
-        if (!password)
-            password = getpass("Proxy password: ");
-
-#endif
-
-        if (!password) {
-            fprintf(stderr, "ERROR: Proxy password missing\n");
-            exit(1);
+    if (!host) {
+        char *newhost = strstr(url, "://");
+        if (newhost) {
+            char *t;
+            newhost += 3;
+            newhost = strdup(newhost);
+            t = newhost + strcspn(newhost, "@/?");
+            if (*t == '@') {
+                newhost = t + 1;
+                t = newhost + strcspn(newhost, "@/?");
+            }
+            *t = '\0';
+            host = newhost;
         }
-        snprintf(buf, BUFSIZ, "%s:%s", user, password);
-        snprintf(buf, BUFSIZ, "Proxy-Authorization: Basic %s\r\n", base64_encode(buf));
-        strcat(msg, buf);
     }
-    if (www_user) {
-        char *user = www_user;
-        char *password = www_password;
-#if HAVE_GETPASS
 
-        if (!password)
-            password = getpass("WWW password: ");
+    if (version[0] == '-' || !version[0] || version[0] == '0') {
+        /* HTTP/0.9, no headers, no version */
+        snprintf(msg, BUFSIZ, "%s %s\r\n", method, url);
+    } else {
+        snprintf(msg, BUFSIZ, "%s %s HTTP/%s\r\n", method, url, version);
 
-#endif
-
-        if (!password) {
-            fprintf(stderr, "ERROR: WWW password missing\n");
-            exit(1);
+        if (host) {
+            snprintf(buf, BUFSIZ, "Host: %s\r\n", host);
+            strcat(msg,buf);
         }
-        snprintf(buf, BUFSIZ, "%s:%s", user, password);
-        snprintf(buf, BUFSIZ, "Authorization: Basic %s\r\n", base64_encode(buf));
-        strcat(msg, buf);
-    }
-    if (keep_alive) {
-        if (port != 80)
-            snprintf(buf, BUFSIZ, "Proxy-Connection: keep-alive\r\n");
-        else
-            snprintf(buf, BUFSIZ, "Connection: keep-alive\r\n");
 
+        if (reload) {
+            snprintf(buf, BUFSIZ, "Pragma: no-cache\r\n");
+            strcat(msg, buf);
+        }
+        if (put_fd > 0) {
+            snprintf(buf, BUFSIZ, "Content-length: %d\r\n", (int) sb.st_size);
+            strcat(msg, buf);
+        }
+        if (opt_noaccept == 0) {
+            snprintf(buf, BUFSIZ, "Accept: */*\r\n");
+            strcat(msg, buf);
+        }
+        if (ims) {
+            snprintf(buf, BUFSIZ, "If-Modified-Since: %s\r\n", mkrfc1123(ims));
+            strcat(msg, buf);
+        }
+        if (max_forwards > -1) {
+            snprintf(buf, BUFSIZ, "Max-Forwards: %d\r\n", max_forwards);
+            strcat(msg, buf);
+        }
+        if (proxy_user) {
+            const char *user = proxy_user;
+            const char *password = proxy_password;
+#if HAVE_GETPASS
+            if (!password)
+                password = getpass("Proxy password: ");
+#endif
+            if (!password) {
+                fprintf(stderr, "ERROR: Proxy password missing\n");
+                exit(1);
+            }
+            snprintf(buf, BUFSIZ, "%s:%s", user, password);
+            snprintf(buf, BUFSIZ, "Proxy-Authorization: Basic %s\r\n", base64_encode(buf));
+            strcat(msg, buf);
+        }
+        if (www_user) {
+            const char *user = www_user;
+            const char *password = www_password;
+#if HAVE_GETPASS
+            if (!password)
+                password = getpass("WWW password: ");
+#endif
+            if (!password) {
+                fprintf(stderr, "ERROR: WWW password missing\n");
+                exit(1);
+            }
+            snprintf(buf, BUFSIZ, "%s:%s", user, password);
+            snprintf(buf, BUFSIZ, "Authorization: Basic %s\r\n", base64_encode(buf));
+            strcat(msg, buf);
+        }
+
+        /* HTTP/1.0 may need keep-alive */
+        if (strcmp(version, "1.0") == 0) {
+            if (keep_alive) {
+                if (strchr(url, ':'))
+                    snprintf(buf, BUFSIZ, "Proxy-Connection: keep-alive\r\n");
+                else
+                    strcat(msg, "Connection: keep-alive\r\n");
+            }
+        } else {
+            if (!keep_alive)
+                strcat(msg, "Connection: close\r\n");
+        }
         strcat(msg, buf);
+
+        strcat(msg, extra_hdrs);
+        strcat(msg, "\r\n");
     }
-    strcat(msg, extra_hdrs);
-    snprintf(buf, BUFSIZ, "\r\n");
-    strcat(msg, buf);
 
     if (opt_verbose)
-        fprintf(stderr, "headers: '%s'\n", msg);
+        fprintf(stderr, "Request: '%s'\n", msg);
 
     if (ping) {
 #if HAVE_SIGACTION
@@ -548,8 +572,8 @@ main(int argc, char *argv[])
         while ((len = myread(conn, buf, sizeof(buf))) > 0) {
             fsize += len;
 
-            if (to_stdout)
-                fwrite(buf, len, 1, stdout);
+            if (to_stdout && fwrite(buf, len, 1, stdout) != 1)
+                perror("client: ERROR writing to stdout");
         }
 
 #ifdef _SQUID_MSWIN_
@@ -612,7 +636,7 @@ main(int argc, char *argv[])
 }
 
 static int
-client_comm_bind(int sock, const IPAddress &addr) {
+client_comm_bind(int sock, const IpAddress &addr) {
 
     int res;
 
@@ -630,7 +654,7 @@ client_comm_bind(int sock, const IPAddress &addr) {
 }
 
 static int
-client_comm_connect(int sock, const IPAddress &addr, struct timeval *tvp) {
+client_comm_connect(int sock, const IpAddress &addr, struct timeval *tvp) {
     int res;
     static struct addrinfo *AI = NULL;
 
